@@ -11,8 +11,13 @@
  */
 package org.gnome.gtk;
 
+import org.gnome.gdk.Color;
+import org.gnome.gdk.Event;
 import org.gnome.gdk.EventExpose;
+import org.gnome.gdk.EventFocus;
 import org.gnome.gdk.EventKey;
+import org.gnome.gdk.EventVisibility;
+import org.gnome.gdk.VisibilityState;
 
 /**
  * The base class of all GTK Widgets. Graphical user interface toolkits have
@@ -31,6 +36,34 @@ public abstract class Widget extends org.gnome.gtk.Object
         super(pointer);
     }
 
+    /**
+     * Cause this Widget to be activated. This has the same effect as when the
+     * user presses the <code>Return</code> key while the Widget is in
+     * focus. Calling this method on a {@link ToggleButton} will toggle its
+     * state, for example. Whatever signals would normally result from the
+     * user activating this Widget by hand will be emitted.
+     * 
+     * <p>
+     * Note that this method only works if this Widget is <var>activatable</var>;
+     * not all are, making this more an optional characteristic that some, but
+     * not all Widgets share.
+     * 
+     * <p>
+     * The Widget must already be showing on the screen for this method to
+     * work (ie, you must invoke {@link #show()} before you can
+     * <code>activate()</code> it). Parent Containers must also have been
+     * shown.
+     * 
+     * @throws UnsupportedOperationException
+     *             if the Widget is not activatable.
+     * @since 4.0.5
+     */
+    public void activate() {
+        if (!GtkWidget.activate(this)) {
+            throw new UnsupportedOperationException();
+        }
+    }
+
     /**
      * Cause this Widget to be mapped to the screen. Flags a widget to be
      * displayed. Any widget that isn't shown will not appear on the screen.
@@ -123,15 +156,15 @@ public abstract class Widget extends org.gnome.gtk.Object
     /**
      * Signal emitted when the focus leaves this Widget. Focus is a concept
      * that is shared evenly between the widget toolkit and the window manager -
-     * which can become apparent if you're wondering <i>why</i> you have lost
-     * focus or regained it.
+     * which often becomes apparent if you're wondering <i>why</i> you have
+     * lost focus or regained it.
      * 
      * @author Andrew Cowie
      * @since 4.0.2
      */
     public interface FOCUS_OUT_EVENT extends GtkWidget.FOCUS_OUT_EVENT
     {
-        public boolean onFocusOutEvent(Widget source, Object event);
+        public boolean onFocusOutEvent(Widget source, EventFocus event);
     }
 
     /**
@@ -304,4 +337,158 @@ public abstract class Widget extends org.gnome.gtk.Object
     public org.gnome.gdk.Window getWindow() {
         return GtkWidgetOverride.getWindow(this);
     }
+
+    /**
+     * Adjust the background colour being used when drawing this Widget. This
+     * leaves all other style properties unchanged.
+     * 
+     * <p>
+     * If you need to change the background colour behind the text in an Entry
+     * or TextView, see
+     * {@link #modifyBase(Widget, StateType Color) modifyBase()}.
+     * 
+     * <p>
+     * This is one of a family of "<code>modify</code>" methods; see
+     * {@link #modifyStyle(Widget, RcStyle) modifyStyle()} for further details
+     * about the interaction of the various theming and style mechanisms.
+     * 
+     * @since 4.0.5
+     */
+    public void modifyBackground(StateType state, Color color) {
+        GtkWidget.modifyBg(this, state, color);
+    }
+
+    /**
+     * A signal providing notification that the Widget has been obscured or
+     * unobscured. To use this, go through the supplied <code>event</code>
+     * parameter to get to the VisibilityState as follows:
+     * 
+     * <pre>
+     * w.connect(new Widget.VISIBILITY_NOTIFY_EVENT() {
+     *     public boolean onVisibilityNotifyEvent(Widget source, EventVisibility event) {
+     *         VisibilityState state = event.getState();
+     *         if (state == VisibilityState.FULLY_OBSCURED) {
+     *            ...
+     *         }
+     *     }
+     *     return false;
+     * });
+     * </pre>
+     * 
+     * See {@link VisibilityState VisibilityState} for the constants
+     * describing the possible three possible changes to an underlying
+     * element's visibility. See also {@link UNMAP_EVENT UNMAP_EVENT} for a
+     * discussion of how this can be used to actively toggle the presentation
+     * of a Window to the user.
+     * 
+     * @author Andrew Cowie
+     * @since 4.0.5
+     */
+    public interface VISIBILITY_NOTIFY_EVENT extends GtkWidget.VISIBILITY_NOTIFY_EVENT
+    {
+        /**
+         * Although this is an <var>event-signal</var>, this merely reports
+         * information coming from the underlying X11 windowing system. Since
+         * a window being obscured by another application's window is not
+         * something you can control (short of requesting the Window holding
+         * this Widget always be on-top), it's not entirely clear what good it
+         * would do to block further emission of this signal. Return
+         * <code>false</code>!
+         */
+        public boolean onVisibilityNotifyEvent(Widget source, EventVisibility event);
+    }
+
+    /**
+     * Connect a <code>VISIBILITY_NOTIFY_EVENT</code> handler.
+     */
+    /*
+     * It turns out that two things are necessary for this signal to work: 1)
+     * GDK_VISIBILITY_NOTIFY_MASK must be set, and to do that 2) the GDK
+     * window must have been assigned. by realize. We do these two steps in
+     * the override.
+     */
+    public void connect(VISIBILITY_NOTIFY_EVENT handler) {
+        GtkWidgetOverride.setEventsVisibility(this);
+        GtkWidget.connect(this, handler);
+    }
+
+    /**
+     * The signal emitted when a Window becomes invisible. This happens in a
+     * variety of scenarios, notably when the Window is minimized, when you
+     * change workspaces, and as a Window is being destroyed.
+     * 
+     * <p>
+     * In combination with
+     * {@link VISIBILITY_NOTIFY_EVENT VISIBILITY_NOTIFY_EVENT}, this can be
+     * used to detect whether a Window is actually currently presented to the
+     * top of the stack and visible to the user:
+     * 
+     * <pre>
+     * private boolean up = false;
+     * ...
+     * final Window w;
+     * final Button b;
+     * ...
+     * w.connect(new Widget.VISIBILITY_NOTIFY_EVENT() {
+     *     public boolean onVisibilityNotifyEvent(Widget source, EventVisibility event) {
+     *         if (event.getState() == VisibilityState.UNOBSCURED) {
+     *             up = true;
+     *         } else {
+     *             up = false;
+     *         }
+     *         return false;
+     *     }
+     * });
+     * 
+     * w.connect(new Widget.UNMAP_EVENT() {
+     *     public boolean onUnmapEvent(Widget source, Event event) {
+     *         up = false;
+     *         return false;
+     *     }
+     * });
+     * </pre>
+     * 
+     * thus allowing you to do something like:
+     * 
+     * <pre>
+     * b.connect(new Button.ACTIVATE() {
+     *     public void onClicked(Button source) {
+     *         if (up) {
+     *             w.hide();
+     *             up = false;
+     *         } else {
+     *             w.present();
+     *             up = true;
+     *         }
+     *     }
+     * }
+     * </pre>
+     * 
+     * to intelligently toggle the visibility of the Window.
+     * 
+     * <p>
+     * Note that you don't need <code>MAP</code> because the the
+     * <code>VISIBILITY_NOTIFY_EVENT</code> will be tripped if you come back
+     * to the workspace the Window is already on.
+     * 
+     * @author Andrew Cowie
+     * @author Ryan Lortie
+     */
+    public interface UNMAP_EVENT extends GtkWidget.UNMAP_EVENT
+    {
+        /**
+         * Although this is an <var>event-signal</var>, this merely reports
+         * information coming from the underlying X11 windowing system. It's
+         * information you can monitor, but don't try to block this signal.
+         * Return <code>false</code>!
+         */
+        boolean onUnmapEvent(Widget source, Event event);
+    }
+
+    /**
+     * Connect a <code>UNMAP_EVENT</code> handler.
+     */
+    public void connect(UNMAP_EVENT handler) {
+        GtkWidget.connect(this, handler);
+    }
 }
